@@ -317,7 +317,7 @@ void gcpFragment::Add(GtkWidget* w)
 		gtk_text_tag_table_add(TextTagTable, tag);
 		g_object_unref((GObject*)tag);
 	}
-	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL));
+	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL)), *chgp;
 	GnomeCanvasItem* item = gnome_canvas_item_new(
 						group,
 						gnome_canvas_rect_ext_get_type(),
@@ -360,46 +360,70 @@ void gcpFragment::Add(GtkWidget* w)
 		}
 		x *= pData->ZoomFactor;
 		y *= pData->ZoomFactor;
-		GtkAnchorType anchor;
-		switch (align)
-		{
-			case -2:
-				y += 6.0;
-				anchor = GTK_ANCHOR_NORTH;
-				break;
-			case -1:
-				x -= 6.0;
-				anchor = GTK_ANCHOR_EAST;
-				break;
-			case 0:
-				anchor = GTK_ANCHOR_CENTER;
-				break;
-			case 1:
-				x += 6.0;
-				anchor = GTK_ANCHOR_WEST;
-				break;
-			case 2:
-				y -= 6.0;
-				anchor = GTK_ANCHOR_SOUTH;
-				break;
+		switch (align) {
+		case -2:
+			x -= ChargeSignSize / 2.;
+			y += ChargeSignHeight / 2.;
+			break;
+		case -1:
+			x -= ChargeSignSize + 2.;
+			break;
+		case 0:
+		case -3:
+			x -= ChargeSignSize / 2.;
+			break;
+		case 1:
+			x += 2.;
+			break;
+		case 2:
+			x -= ChargeSignSize / 2.;
+			y -= ChargeSignHeight / 2.;
+			break;
 		}
-		char *markup;
-		if 	(abs (charge) > 1)
-			markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-		else
-			markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-		item = gnome_canvas_item_new(
-						group,
-						gnome_canvas_text_ext_get_type(),
-						"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-						"font-desc", pData->View->GetPangoSmallFontDesc (),
-						"markup", markup,
-						"anchor", anchor,
-						"x", x,
-						"y", y,
-						NULL);
-		g_free (markup);
-		g_object_set_data ((GObject*)group, "charge", item);
+		y -= ChargeSignHeight / 2. - ChargeYAlign + ChargeSignSize - 1.;
+		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
+					group,
+					gnome_canvas_group_ext_get_type(),
+					NULL);
+		g_object_set_data (G_OBJECT (group), "charge", chgp);
+		item = gnome_canvas_item_new (
+					chgp,
+					gnome_canvas_ellipse_ext_get_type (),
+					"x1", x,
+					"y1", y,
+					"x2", x + ChargeSignSize,
+					"y2", y + ChargeSignSize,
+					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+					"width_units", 0.5,
+					NULL
+				);
+		g_object_set_data (G_OBJECT (group), "circle", item);
+		ArtBpath *path = art_new (ArtBpath, 5);
+		path[0].code = ART_MOVETO_OPEN;
+		path[0].x3 = x + 1.;
+		path[1].code = ART_LINETO;
+		path[1].x3 = x + ChargeSignSize - 1.;
+		path[0].y3 = path[1].y3 = y + ChargeSignSize / 2.;
+		if (charge > 0) {
+			path[2].code = ART_MOVETO_OPEN;
+			path[2].y3 = y + 1.;
+			path[3].code = ART_LINETO;
+			path[3].y3 = y + ChargeSignSize - 1.;
+			path[2].x3 = path[3].x3 = x + ChargeSignSize / 2.;
+			path[4].code = ART_END;
+		} else
+			path[2].code = ART_END;
+		GnomeCanvasPathDef *cpd = gnome_canvas_path_def_new_from_bpath (path);
+		item = gnome_canvas_item_new (
+					chgp,
+					gnome_canvas_bpath_ext_get_type (),
+					"bpath", cpd,
+					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+					"width_units", 1.,
+					NULL
+				);
+		gnome_canvas_path_def_unref (cpd);
+		g_object_set_data (G_OBJECT (group), "sign", item);
 	}
 	pData->Items[this] = group;
 	gpointer obj = g_object_get_data (G_OBJECT (group), "fragment");
@@ -420,9 +444,11 @@ void gcpFragment::SetSelected(GtkWidget* w, int state)
 	}
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
-	gpointer item = g_object_get_data (G_OBJECT (group), "charge");
-	if (item)
-		g_object_set(item, "fill_color", chargecolor, NULL);
+	gpointer item;
+	if (item = g_object_get_data(G_OBJECT(group), "circle"))
+		g_object_set (item, "outline_color", chargecolor, NULL);
+	if (item = g_object_get_data(G_OBJECT(group), "sign"))
+		g_object_set (item, "outline_color", chargecolor, NULL);
 }
 
 void gcpFragment::Update(GtkWidget* w)
@@ -454,38 +480,58 @@ void gcpFragment::Update(GtkWidget* w)
 			}
 			x *= pData->ZoomFactor;
 			y *= pData->ZoomFactor;
-			GtkAnchorType anchor;
-			switch (align)
-			{
-				case -2:
-					anchor = GTK_ANCHOR_NORTH;
-					break;
-				case -1:
-					anchor = GTK_ANCHOR_EAST;
-					break;
-				case 0:
-					anchor = GTK_ANCHOR_CENTER;
-					break;
-				case 1:
-					anchor = GTK_ANCHOR_WEST;
-					break;
-				case 2:
-					anchor = GTK_ANCHOR_SOUTH;
-					break;
+			switch (align) {
+			case -2:
+				x -= ChargeSignSize / 2.;
+				y += ChargeSignSize / 2.;
+				break;
+			case -1:
+				x -= ChargeSignSize + 2.;
+				break;
+			case 0:
+			case -3:
+				x -= ChargeSignSize / 2.;
+				break;
+			case 1:
+				x += 2.;
+				break;
+			case 2:
+				x -= ChargeSignSize / 2.;
+				y -= ChargeSignSize / 2.;
+				break;
 			}
-			char *markup;
-			if 	(abs (charge) > 1)
-				markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-			else
-				markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
+			y -= ChargeSignHeight / 2. - ChargeYAlign + ChargeSignSize - 1.;
+			item = g_object_get_data (G_OBJECT (group), "circle");
 			g_object_set (G_OBJECT (item),
-						"markup", markup,
-						"anchor", anchor,
-						"x", x,
-						"y", y,
+						"x1", x,
+						"y1", y,
+						"x2", x + ChargeSignSize,
+						"y2", y + ChargeSignSize,
 						NULL);
-			g_free (markup);
+			item = g_object_get_data (G_OBJECT (group), "sign");
+			ArtBpath *path = art_new (ArtBpath, 5);
+			path[0].code = ART_MOVETO_OPEN;
+			path[0].x3 = x + 1.;
+			path[1].code = ART_LINETO;
+			path[1].x3 = x + ChargeSignSize - 1.;
+			path[0].y3 = path[1].y3 = y + ChargeSignSize / 2.;
+			if (charge > 0) {
+				path[2].code = ART_MOVETO_OPEN;
+				path[2].y3 = y + 1.;
+				path[3].code = ART_LINETO;
+				path[3].y3 = y + ChargeSignSize - 1.;
+				path[2].x3 = path[3].x3 = x + ChargeSignSize / 2.;
+				path[4].code = ART_END;
+			} else
+				path[2].code = ART_END;
+			GnomeCanvasPathDef *cpd = gnome_canvas_path_def_new_from_bpath (path);
+			g_object_set (G_OBJECT (item),
+						"bpath", cpd,
+						NULL
+					);
+			gnome_canvas_path_def_unref (cpd);
 		} else {
+			GnomeCanvasGroup *chgp;
 			int align = GetChargePosition(m_Atom, Pos, Angle, x, y);
 			x *= pData->ZoomFactor;
 			if (Dist != 0.) {
@@ -493,48 +539,76 @@ void gcpFragment::Update(GtkWidget* w)
 				y = m_y - Dist * sin (Angle);
 			}
 			y *= pData->ZoomFactor;
-			GtkAnchorType anchor;
-			switch (align)
-			{
-				case -2:
-					anchor = GTK_ANCHOR_NORTH;
-					break;
-				case -1:
-					anchor = GTK_ANCHOR_EAST;
-					break;
-				case 0:
-					anchor = GTK_ANCHOR_CENTER;
-					break;
-				case 1:
-					anchor = GTK_ANCHOR_WEST;
-					break;
-				case 2:
-					anchor = GTK_ANCHOR_SOUTH;
-					break;
+			switch (align) {
+			case -2:
+				x -= ChargeSignSize / 2.;
+				y += ChargeSignSize / 2.;
+				break;
+			case -1:
+				x -= ChargeSignSize + 2.;
+				break;
+			case 0:
+			case -3:
+				x -= ChargeSignSize / 2.;
+				break;
+			case 1:
+				x += 2.;
+				break;
+			case 2:
+				x -= ChargeSignSize / 2.;
+				y -= ChargeSignSize / 2.;
+				break;
 			}
-			char *markup;
-			if 	(abs (charge) > 1)
-				markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-			else
-				markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-			item = gnome_canvas_item_new(
-							group,
-							gnome_canvas_text_ext_get_type(),
-							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-							"font-desc", pData->View->GetPangoSmallFontDesc (),
-							"markup", markup,
-							"anchor", anchor,
-							"x", x,
-							"y", y,
-							NULL);
-			g_free (markup);
-			g_object_set_data ((GObject*)group, "charge", item);
+			y -= ChargeSignHeight / 2. - ChargeYAlign + ChargeSignSize - 1.;
+			chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
+						group,
+						gnome_canvas_group_ext_get_type(),
+						NULL);
+			g_object_set_data (G_OBJECT (group), "charge", chgp);
+			item = gnome_canvas_item_new (
+						chgp,
+						gnome_canvas_ellipse_ext_get_type (),
+						"x1", x,
+						"y1", y,
+						"x2", x + ChargeSignSize,
+						"y2", y + ChargeSignSize,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"width_units", 0.5,
+						NULL
+					);
+			g_object_set_data (G_OBJECT (group), "circle", item);
+			ArtBpath *path = art_new (ArtBpath, 5);
+			path[0].code = ART_MOVETO_OPEN;
+			path[0].x3 = x + 1.;
+			path[1].code = ART_LINETO;
+			path[1].x3 = x + ChargeSignSize - 1.;
+			path[0].y3 = path[1].y3 = y + ChargeSignSize / 2.;
+			if (charge > 0) {
+				path[2].code = ART_MOVETO_OPEN;
+				path[2].y3 = y + 1.;
+				path[3].code = ART_LINETO;
+				path[3].y3 = y + ChargeSignSize - 1.;
+				path[2].x3 = path[3].x3 = x + ChargeSignSize / 2.;
+				path[4].code = ART_END;
+			} else
+				path[2].code = ART_END;
+			GnomeCanvasPathDef *cpd = gnome_canvas_path_def_new_from_bpath (path);
+			item = gnome_canvas_item_new (
+						chgp,
+						gnome_canvas_bpath_ext_get_type (),
+						"bpath", cpd,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"width_units", 1.,
+						NULL
+					);
+			gnome_canvas_path_def_unref (cpd);
+			g_object_set_data (G_OBJECT (group), "sign", item);
 		}
-		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
 	} else {
-		if (item)
+		if (item) {
 			gtk_object_destroy (GTK_OBJECT (item));
 			g_object_set_data ((GObject*)group, "charge", NULL);
+		}
 	}
 }
 
@@ -928,7 +1002,8 @@ Object* gcpFragment::GetAtomAt(double x, double y, double z)
 	gcpDocument* pDoc = (gcpDocument*)GetDocument();
 	gcpApplication* pApp = pDoc->GetApplication();
 	if (pApp->GetActiveTool() == pApp->GetTool("Fragment")) return NULL;
-	if (m_Atom->GetBondsNumber()) return m_Atom;
+	if (m_Atom->GetBondsNumber () || m_Atom->GetCharge ())
+		return m_Atom;
 	if (!pDoc) return NULL;
 	GtkWidget* pWidget = pDoc->GetView()->GetWidget();
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(pWidget), "data");
@@ -1081,6 +1156,7 @@ int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, unsigned char& Pos, d
 			return 0;
 	} else
 		return 0;
+
 	switch (Pos) {
 	case CHARGE_NE:
 		x = m_x + width / 2.0;
